/*
 * linux/arch/arm/mach-omap2/board-blueshark.c
 *
 * Copyright (C) 2008 Texas Instruments
 *
 * Modified from mach-omap2/board-3430sdp.c
 *
 * Initial code: Syed Mohammed Khasim
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/io.h>
#include <linux/leds.h>
#include <linux/gpio.h>
#include <linux/irq.h>
#include <linux/input.h>
#include <linux/gpio_keys.h>
#include <linux/spi/spi.h>

#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/nand.h>

#include <linux/regulator/machine.h>
#include <linux/i2c/twl.h>

#include <mach/hardware.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <plat/board.h>
#include <plat/common.h>
#include <plat/display.h>
#include <plat/gpmc.h>
#include <plat/nand.h>
#include <plat/usb.h>
#include <plat/timer-gp.h>
#include <plat/clock.h>
#include <plat/omap-pm.h>
#include <plat/mcspi.h>

#include "mux.h"
#include "mmc-twl4030.h"
#include "pm.h"
#include "omap3-opp.h"

#ifdef CONFIG_PM
static struct omap_opp * _omap35x_mpu_rate_table        = omap35x_mpu_rate_table;
static struct omap_opp * _omap37x_mpu_rate_table        = omap37x_mpu_rate_table;
static struct omap_opp * _omap35x_dsp_rate_table        = omap35x_dsp_rate_table;
static struct omap_opp * _omap37x_dsp_rate_table        = omap37x_dsp_rate_table;
static struct omap_opp * _omap35x_l3_rate_table         = omap35x_l3_rate_table;
static struct omap_opp * _omap37x_l3_rate_table         = omap37x_l3_rate_table;
#else   /* CONFIG_PM */
static struct omap_opp * _omap35x_mpu_rate_table        = NULL;
static struct omap_opp * _omap37x_mpu_rate_table        = NULL;
static struct omap_opp * _omap35x_dsp_rate_table        = NULL;
static struct omap_opp * _omap37x_dsp_rate_table        = NULL;
static struct omap_opp * _omap35x_l3_rate_table         = NULL;
static struct omap_opp * _omap37x_l3_rate_table         = NULL;
#endif  /* CONFIG_PM */


#define GPMC_CS0_BASE  0x60
#define GPMC_CS_SIZE   0x30

#define NAND_BLOCK_SIZE		SZ_128K

#if defined(CONFIG_KS8851) || defined(CONFIG_KS8851_MODULE)

#include <plat/mcspi.h>
#include <linux/spi/spi.h>

#define BLUESHARK_GPIO_KS8851_IRQ 186

static struct omap2_mcspi_device_config ks8851_spi_chip_info = {
	.turbo_mode	= 0,
	.single_channel	= 1,	/* 0: slave, 1: master */
};

static struct spi_board_info blueshark_spi_ks8851_info[] __initdata = {
	{
		.modalias		= "ks8851",
		.bus_num		= 3,
		.chip_select		= 0,
		.max_speed_hz		= 36000000,
		.controller_data	= &ks8851_spi_chip_info,
	},
};

static void __init blueshark_ks8851_init(void)
{
	if ((gpio_request(BLUESHARK_GPIO_KS8851_IRQ, "KS8851_IRQ") == 0) &&
	    (gpio_direction_input(BLUESHARK_GPIO_KS8851_IRQ) == 0)) {
		gpio_export(BLUESHARK_GPIO_KS8851_IRQ, 0);
		blueshark_spi_ks8851_info[0].irq = OMAP_GPIO_IRQ(BLUESHARK_GPIO_KS8851_IRQ);
		set_irq_type(blueshark_spi_ks8851_info[0].irq, IRQ_TYPE_EDGE_FALLING);
	} else {
		printk(KERN_ERR "could not obtain gpio for KS8851_IRQ\n");
		return;
	}

	spi_register_board_info(blueshark_spi_ks8851_info,
			ARRAY_SIZE(blueshark_spi_ks8851_info));
}

#else
static inline void __init blueshark_ks8851_init(void) { return; }
#endif

#if defined(CONFIG_TOUCHSCREEN_ADS7846) || \
	defined(CONFIG_TOUCHSCREEN_ADS7846_MODULE)

#include <plat/mcspi.h>
#include <linux/spi/spi.h>
#include <linux/spi/ads7846.h>

#define BLUESHARK_GPIO_PENDOWN	10

static struct omap2_mcspi_device_config ads7846_mcspi_config = {
	.turbo_mode	= 0,
	.single_channel	= 1,	/* 0: slave, 1: master */
};

static int ads7846_get_pendown_state(void)
{
	return !gpio_get_value(BLUESHARK_GPIO_PENDOWN);
}

static struct ads7846_platform_data ads7846_config = {
	.x_max			= 0x0fff,
	.y_max			= 0x0fff,
	.x_plate_ohms		= 300,
	.y_plate_ohms		= 180,
	.pressure_max		= 255,
	.debounce_max		= 10,
	.debounce_tol		= (~0),
	.debounce_rep		= 0,
	.get_pendown_state	= ads7846_get_pendown_state,
	.keep_vref_on		= 1,
};

static struct spi_board_info blueshark_spi_ads7846_info[] __initdata = {
	{
		.modalias		= "ads7846",
		.bus_num		= 1,
		.chip_select		= 0,
		.max_speed_hz		= 1500000,
		.controller_data	= &ads7846_mcspi_config,
		.irq			= OMAP_GPIO_IRQ(BLUESHARK_GPIO_PENDOWN),
		.platform_data		= &ads7846_config,
	},
};

static void __init blueshark_ads7846_init(void)
{
	if ((gpio_request(BLUESHARK_GPIO_PENDOWN, "ADS7846_PENDOWN") == 0) &&
	    (gpio_direction_input(BLUESHARK_GPIO_PENDOWN) == 0)) {
		gpio_export(BLUESHARK_GPIO_PENDOWN, 0);
	} else {
		printk(KERN_ERR "could not obtain gpio for ADS7846_PENDOWN\n");
		return;
	}

	spi_register_board_info(blueshark_spi_ads7846_info,
			ARRAY_SIZE(blueshark_spi_ads7846_info));
}

#else
static inline void __init blueshark_ads7846_init(void) { return; }
#endif

#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)

#include <linux/smsc911x.h>
#define BLUESHARK_SMSC911X_CS	5
#define BLUESHARK_SMSC911X_GPIO	17

static struct resource blueshark_smsc911x_resources[] = {
	{
		.name	= "smsc911x-memory",
		.flags	= IORESOURCE_MEM,
	},
	{
		.flags	= IORESOURCE_IRQ | IORESOURCE_IRQ_LOWLEVEL,
	},
};

static struct smsc911x_platform_config blueshark_smsc911x_config = {
	.irq_polarity	= SMSC911X_IRQ_POLARITY_ACTIVE_LOW,
	.irq_type	= SMSC911X_IRQ_TYPE_OPEN_DRAIN,
	.flags		= SMSC911X_USE_32BIT ,
	.phy_interface	= PHY_INTERFACE_MODE_MII,
};

static struct platform_device blueshark_smsc911x_device = {
	.name		= "smsc911x",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(blueshark_smsc911x_resources),
	.resource	= blueshark_smsc911x_resources,
	.dev		= {
		.platform_data = &blueshark_smsc911x_config,
	},
};

static inline void __init blueshark_init_smsc911x(void)
{
	unsigned long cs_mem_base;

	if (gpmc_cs_request(BLUESHARK_SMSC911X_CS, SZ_16M, &cs_mem_base) < 0) {
		printk(KERN_ERR "Failed request for GPMC mem for smsc911x\n");
		return;
	}

	blueshark_smsc911x_resources[0].start = cs_mem_base + 0x0;
	blueshark_smsc911x_resources[0].end   = cs_mem_base + 0xff;

	if ((gpio_request(BLUESHARK_SMSC911X_GPIO, "SMSC911X IRQ") == 0) &&
	    (gpio_direction_input(BLUESHARK_SMSC911X_GPIO) == 0)) {
		gpio_export(BLUESHARK_SMSC911X_GPIO, 0);
	} else {
		printk(KERN_ERR "could not obtain gpio for SMSC911X IRQ\n");
		return;
	}

	blueshark_smsc911x_resources[1].start = OMAP_GPIO_IRQ(BLUESHARK_SMSC911X_GPIO);
	blueshark_smsc911x_resources[1].end = 0;

	platform_device_register(&blueshark_smsc911x_device);
}

#else
static inline void __init blueshark_init_smsc911x(void) { return; }
#endif

static struct mtd_partition blueshark_nand_partitions[] = {
	/* All the partition sizes are listed in terms of NAND block size */
	{
		.name		= "X-Loader",
		.offset		= 0,
		.size		= 4 * NAND_BLOCK_SIZE,
		.mask_flags	= MTD_WRITEABLE,	/* force read-only */
	},
	{
		.name		= "U-Boot",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x80000 */
		.size		= 15 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "U-Boot Env",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x260000 */
		.size		= 1 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "Kernel",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x280000 */
		.size		= 32 * NAND_BLOCK_SIZE,
	},
	{
		.name		= "File System",
		.offset		= MTDPART_OFS_APPEND,	/* Offset = 0x680000 */
		.size		= MTDPART_SIZ_FULL,
	},
};

static struct omap_nand_platform_data blueshark_nand_data = {
	.options	= NAND_BUSWIDTH_16,
	.parts		= blueshark_nand_partitions,
	.nr_parts	= ARRAY_SIZE(blueshark_nand_partitions),
	.dma_channel	= -1,		/* disable DMA in OMAP NAND driver */
	.nand_setup	= NULL,
	.dev_ready	= NULL,
};

static struct resource blueshark_nand_resource = {
	.flags		= IORESOURCE_MEM,
};

static struct platform_device blueshark_nand_device = {
	.name		= "omap2-nand",
	.id		= -1,
	.dev		= {
		.platform_data	= &blueshark_nand_data,
	},
	.num_resources	= 1,
	.resource	= &blueshark_nand_resource,
};

/* DSS */

#define BLUESHARK_GPIO_LCD_RST 12
#define BLUESHARK_GPIO_LCD_SHD 13
#define BLUESHARK_GPIO_LCD_BL 1
#define BLUESHARK_GPIO_DVI_EN 170

static int blueshark_enable_disp(struct omap_dss_device *dssdev)
{
	if (dssdev->reset_gpio != -1)
		gpio_set_value(dssdev->reset_gpio, 1);

#if defined(CONFIG_PANEL_PT35) || defined(CONFIG_PANEL_PT35_MODULE)
	udelay(100);
	gpio_set_value(BLUESHARK_GPIO_LCD_SHD, 0);
#endif
	return 0;
}

static void blueshark_disable_disp(struct omap_dss_device *dssdev)
{
	if (dssdev->reset_gpio != -1)
		gpio_set_value(dssdev->reset_gpio, 0);
}

static struct omap_dss_device blueshark_dvi_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "dvi",
	.driver_name = "generic_panel",
	.phy.dpi.data_lines = 24,
	.reset_gpio = BLUESHARK_GPIO_DVI_EN,
	.platform_enable = blueshark_enable_disp,
	.platform_disable = blueshark_disable_disp,
};

#if defined(CONFIG_PANEL_MTF7) || defined(CONFIG_PANEL_MTF7_MODULE)
static struct omap_dss_device blueshark_mtf7_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "mtf7",
	.driver_name = "microtips_mtf7_panel",
	.phy.dpi.data_lines = 18,
	.reset_gpio = BLUESHARK_GPIO_LCD_BL,
	.platform_enable = blueshark_enable_disp,
	.platform_disable = blueshark_disable_disp,
};
#endif

#if defined(CONFIG_PANEL_OSD8) || defined(CONFIG_PANEL_OSD8_MODULE)
static struct omap_dss_device blueshark_osd8_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "osd8",
	.driver_name = "osd_osd8_panel",
	.phy.dpi.data_lines = 18,
	.reset_gpio = BLUESHARK_GPIO_LCD_BL,
	.platform_enable = blueshark_enable_disp,
	.platform_disable = blueshark_disable_disp,
};
#endif

#if defined(CONFIG_PANEL_OSD102) || defined(CONFIG_PANEL_OSD102_MODULE)
static struct omap_dss_device blueshark_osd102_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "osd102",
	.driver_name = "osd_osd102_panel",
	.phy.dpi.data_lines = 18,
	.reset_gpio = BLUESHARK_GPIO_LCD_BL,
	.platform_enable = blueshark_enable_disp,
	.platform_disable = blueshark_disable_disp,
};
#endif

#if defined(CONFIG_PANEL_PT35) || defined(CONFIG_PANEL_PT35_MODULE)
static struct omap_dss_device blueshark_pt35_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "pt35",
	.driver_name = "pt35_panel",
	.phy.dpi.data_lines = 18,
	.reset_gpio = BLUESHARK_GPIO_LCD_BL,
	.platform_enable = blueshark_enable_disp,
	.platform_disable = blueshark_disable_disp,
};
#endif

static int blueshark_panel_enable_tv(struct omap_dss_device *dssdev)
{
#define ENABLE_VDAC_DEDICATED           0x03
#define ENABLE_VDAC_DEV_GRP             0x20

	twl_i2c_write_u8(TWL4030_MODULE_PM_RECEIVER,
			ENABLE_VDAC_DEDICATED,
			TWL4030_VDAC_DEDICATED);
	twl_i2c_write_u8(TWL4030_MODULE_PM_RECEIVER,
			ENABLE_VDAC_DEV_GRP, TWL4030_VDAC_DEV_GRP);

	return 0;
}

static void blueshark_panel_disable_tv(struct omap_dss_device *dssdev)
{
	twl_i2c_write_u8(TWL4030_MODULE_PM_RECEIVER, 0x00,
			TWL4030_VDAC_DEDICATED);
	twl_i2c_write_u8(TWL4030_MODULE_PM_RECEIVER, 0x00,
			TWL4030_VDAC_DEV_GRP);
}

static struct omap_dss_device blueshark_tv_device = {
	.name = "tv",
	.driver_name = "venc",
	.type = OMAP_DISPLAY_TYPE_VENC,
	.phy.venc.type = OMAP_DSS_VENC_TYPE_SVIDEO,
	.platform_enable = blueshark_panel_enable_tv,
	.platform_disable = blueshark_panel_disable_tv,
};

static struct omap_dss_device *blueshark_dss_devices[] = {
	&blueshark_dvi_device,
	&blueshark_tv_device,
#if defined(CONFIG_PANEL_MTF7) || defined(CONFIG_PANEL_MTF7_MODULE)
	&blueshark_mtf7_device,
#endif
#if defined(CONFIG_PANEL_OSD8) || defined(CONFIG_PANEL_OSD8_MODULE)
	&blueshark_osd8_device,
#endif
#if defined(CONFIG_PANEL_OSD102) || defined(CONFIG_PANEL_OSD102_MODULE)
	&blueshark_osd102_device,
#endif
#if defined(CONFIG_PANEL_PT35) || defined(CONFIG_PANEL_PT35_MODULE)
	&blueshark_pt35_device,
#endif
};

static struct omap_dss_board_info blueshark_dss_data = {
	.num_devices = ARRAY_SIZE(blueshark_dss_devices),
	.devices = blueshark_dss_devices,
	.default_device = &blueshark_dvi_device,
};

static struct platform_device blueshark_dss_device = {
	.name          = "omapdss",
	.id            = -1,
	.dev            = {
		.platform_data = &blueshark_dss_data,
	},
};

static struct regulator_consumer_supply blueshark_vdac_supply = {
	.supply		= "vdda_dac",
	.dev		= &blueshark_dss_device.dev,
};

static struct regulator_consumer_supply blueshark_vdvi_supply = {
	.supply		= "vdds_dsi",
	.dev		= &blueshark_dss_device.dev,
};

static void __init blueshark_display_init(void)
{
	gpio_set_value(blueshark_dvi_device.reset_gpio, 0);

#if defined(CONFIG_PANEL_PT35) || defined(CONFIG_PANEL_PT35_MODULE)
	omap_mux_init_gpio(BLUESHARK_GPIO_LCD_RST, OMAP_PIN_OUTPUT);
	if (gpio_request(BLUESHARK_GPIO_LCD_RST, "LCD_RST")) printk("Can't requiest BLUESHARK_GPIO_LCD_RST\n");
	gpio_direction_output(BLUESHARK_GPIO_LCD_RST, 0);
	omap_mux_init_gpio(BLUESHARK_GPIO_LCD_SHD, OMAP_PIN_INPUT);
	if (gpio_request(BLUESHARK_GPIO_LCD_SHD, "LCD_SHD")) printk("Can't requiest BLUESHARK_GPIO_LCD_SHD\n");
	gpio_direction_output(BLUESHARK_GPIO_LCD_SHD, 1);

	mdelay(120);
	gpio_set_value(BLUESHARK_GPIO_LCD_RST, 1);
#endif
}

#include "sdram-micron-mt46h32m32lf-6.h"

static struct twl4030_hsmmc_info mmc[] = {
	{
		.mmc		= 1,
		.wires		= 4,
		.gpio_wp	= 29,
	},
	{
		.mmc		= 2,
		.wires		= 4,
		.transceiver	= true,
		.ocr_mask	= 0x00100000,	/* 3.3V */
	},
	{}	/* Terminator */
};

static struct regulator_consumer_supply blueshark_vmmc1_supply = {
	.supply			= "vmmc",
};

static struct gpio_led gpio_leds[];

static int blueshark_twl_gpio_setup(struct device *dev,
		unsigned gpio, unsigned ngpio)
{
	if (system_rev >= 0x20 && system_rev <= 0x34301000) {
		omap_mux_init_gpio(23, OMAP_PIN_INPUT);
		mmc[0].gpio_wp = 23;
	} else {
		omap_mux_init_gpio(29, OMAP_PIN_INPUT);
	}
	/* gpio + 0 is "mmc0_cd" (input/IRQ) */
	mmc[0].gpio_cd = gpio + 0;
	twl4030_mmc_init(mmc);

	/* link regulators to MMC adapters */
	blueshark_vmmc1_supply.dev = mmc[0].dev;

	/* REVISIT: need ehci-omap hooks for external VBUS
	 * power switch and overcurrent detect
	 */

	if (cpu_is_omap3630()) {
		/* Power on DVI, Serial and PWR led */ 
 		gpio_request(gpio + 1, "nDVI_PWR_EN");
		gpio_direction_output(gpio + 1, 0);	

		/* Power on camera interface */
		gpio_request(gpio + 2, "CAM_EN");
		gpio_direction_output(gpio + 2, 1);

		/* TWL4030_GPIO_MAX + 0 == ledA, EHCI nEN_USB_PWR (out, active low) */
		gpio_request(gpio + TWL4030_GPIO_MAX, "nEN_USB_PWR");
		gpio_direction_output(gpio + TWL4030_GPIO_MAX, 1);
	}
	else {
		gpio_request(gpio + 1, "EHCI_nOC");
		gpio_direction_input(gpio + 1);

		/* TWL4030_GPIO_MAX + 0 == ledA, EHCI nEN_USB_PWR (out, active low) */
		gpio_request(gpio + TWL4030_GPIO_MAX, "nEN_USB_PWR");
		gpio_direction_output(gpio + TWL4030_GPIO_MAX, 0);
	}


	/* TWL4030_GPIO_MAX + 1 == ledB, PMU_STAT (out, active low LED) */
	gpio_leds[0].gpio = gpio + TWL4030_GPIO_MAX + 1;

	return 0;
}

static struct twl4030_gpio_platform_data blueshark_gpio_data = {
	.gpio_base	= OMAP_MAX_GPIO_LINES,
	.irq_base	= TWL4030_GPIO_IRQ_BASE,
	.irq_end	= TWL4030_GPIO_IRQ_END,
	.use_leds	= true,
	.pullups	= BIT(1),
	.pulldowns	= BIT(2) | BIT(6) | BIT(7) | BIT(8) | BIT(13)
				| BIT(15) | BIT(16) | BIT(17),
	.setup		= blueshark_twl_gpio_setup,
};

/* VMMC1 for MMC1 pins CMD, CLK, DAT0..DAT3 (20 mA, plus card == max 220 mA) */
static struct regulator_init_data blueshark_vmmc1 = {
	.constraints = {
		.min_uV			= 1850000,
		.max_uV			= 3150000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_VOLTAGE
					| REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &blueshark_vmmc1_supply,
};

/* VDAC for DSS driving S-Video (8 mA unloaded, max 65 mA) */
static struct regulator_init_data blueshark_vdac = {
	.constraints = {
		.min_uV			= 1800000,
		.max_uV			= 1800000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &blueshark_vdac_supply,
};

/* VPLL2 for digital video outputs */
static struct regulator_init_data blueshark_vpll2 = {
	.constraints = {
		.name			= "VDVI",
		.min_uV			= 1800000,
		.max_uV			= 1800000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL
					| REGULATOR_MODE_STANDBY,
		.valid_ops_mask		= REGULATOR_CHANGE_MODE
					| REGULATOR_CHANGE_STATUS,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &blueshark_vdvi_supply,
};

static struct twl4030_usb_data blueshark_usb_data = {
	.usb_mode	= T2_USB_MODE_ULPI,
};

static struct twl4030_codec_audio_data blueshark_audio_data = {
	.audio_mclk = 26000000,
};

static struct twl4030_codec_data blueshark_codec_data = {
	.audio_mclk = 26000000,
	.audio = &blueshark_audio_data,
};

static struct twl4030_madc_platform_data blueshark_madc_data = {
	.irq_line	= 1,
};

static struct twl4030_platform_data blueshark_twldata = {
	.irq_base	= TWL4030_IRQ_BASE,
	.irq_end	= TWL4030_IRQ_END,

	/* platform_data for children goes here */
	.usb		= &blueshark_usb_data,
	.gpio		= &blueshark_gpio_data,
	.codec		= &blueshark_codec_data,
	.madc		= &blueshark_madc_data,
	.vmmc1		= &blueshark_vmmc1,
	.vdac		= &blueshark_vdac,
	.vpll2		= &blueshark_vpll2,
};

static struct i2c_board_info __initdata blueshark_i2c1_boardinfo[] = {
	{
		I2C_BOARD_INFO("twl4030", 0x48),
		.flags = I2C_CLIENT_WAKE,
		.irq = INT_34XX_SYS_NIRQ,
		.platform_data = &blueshark_twldata,
	},
};

	
#if defined(CONFIG_EEPROM_AT24) || defined(CONFIG_EEPROM_AT24_MODULE)
#include <linux/i2c/at24.h>

static struct at24_platform_data m24c01 = {
	        .byte_len       = SZ_1K / 4,
	        .page_size      = 16,
};

static struct i2c_board_info __initdata blueshark_i2c2_boardinfo[] = {
	{
		I2C_BOARD_INFO("24c01", 0x50),
		.platform_data  = &m24c01,
	},
};
#else
static struct i2c_board_info __initdata blueshark_i2c2_boardinfo[] = {};
#endif

static int __init blueshark_i2c_init(void)
{
	omap_register_i2c_bus(1, 2600, blueshark_i2c1_boardinfo,
			ARRAY_SIZE(blueshark_i2c1_boardinfo));
	omap_register_i2c_bus(2, 400,  blueshark_i2c2_boardinfo,
			ARRAY_SIZE(blueshark_i2c2_boardinfo));
	/* Bus 3 is attached to the DVI port where devices like the pico DLP
	 * projector don't work reliably with 400kHz */
	omap_register_i2c_bus(3, 100, NULL, 0);
	return 0;
}

static struct gpio_led gpio_leds[] = {
	{
		.name			= "beagleboard::pmu_stat",
		.gpio			= -EINVAL,	/* gets replaced */
		.active_low		= true,
	},
};

static struct gpio_led_platform_data gpio_led_info = {
	.leds		= gpio_leds,
	.num_leds	= ARRAY_SIZE(gpio_leds),
};

static struct platform_device leds_gpio = {
	.name	= "leds-gpio",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_led_info,
	},
};

static struct gpio_keys_button gpio_buttons[] = {
	{
		.code			= BTN_EXTRA,
		.gpio			= 7,
		.desc			= "user",
		.wakeup			= 1,
	},
};

static struct gpio_keys_platform_data gpio_key_info = {
	.buttons	= gpio_buttons,
	.nbuttons	= ARRAY_SIZE(gpio_buttons),
};

static struct platform_device keys_gpio = {
	.name	= "gpio-keys",
	.id	= -1,
	.dev	= {
		.platform_data	= &gpio_key_info,
	},
};

static int __init blueshark_spi_init(void)
{
	blueshark_ads7846_init();
	blueshark_ks8851_init();
	return 0;
}

static void __init blueshark_init_irq(void)
{
        if (cpu_is_omap3630())
        {
                omap2_init_common_hw(mt46h32m32lf6_sdrc_params,
                                        mt46h32m32lf6_sdrc_params,
                                        _omap37x_mpu_rate_table,
                                        _omap37x_dsp_rate_table,
                                        _omap37x_l3_rate_table);
        }
        else
        {
                omap2_init_common_hw(mt46h32m32lf6_sdrc_params,
                                        mt46h32m32lf6_sdrc_params,
                                        _omap35x_mpu_rate_table,
                                        _omap35x_dsp_rate_table,
                                        _omap35x_l3_rate_table);
        }
	omap_init_irq();
#ifdef CONFIG_OMAP_32K_TIMER
	omap2_gp_clockevent_set_gptimer(12);
#endif
	omap_gpio_init();
}

static struct platform_device *blueshark_devices[] __initdata = {
	&leds_gpio,
	&keys_gpio,
	&blueshark_dss_device,
};

static void __init blueshark_flash_init(void)
{
	u8 cs = 0;
	u8 nandcs = GPMC_CS_NUM + 1;

	u32 gpmc_base_add = OMAP34XX_GPMC_VIRT;

	/* find out the chip-select on which NAND exists */
	while (cs < GPMC_CS_NUM) {
		u32 ret = 0;
		ret = gpmc_cs_read_reg(cs, GPMC_CS_CONFIG1);

		if ((ret & 0xC00) == 0x800) {
			printk(KERN_INFO "Found NAND on CS%d\n", cs);
			if (nandcs > GPMC_CS_NUM)
				nandcs = cs;
		}
		cs++;
	}

	if (nandcs > GPMC_CS_NUM) {
		printk(KERN_INFO "NAND: Unable to find configuration "
				 "in GPMC\n ");
		return;
	}

	if (nandcs < GPMC_CS_NUM) {
		blueshark_nand_data.cs = nandcs;
		blueshark_nand_data.gpmc_cs_baseaddr = (void *)
			(gpmc_base_add + GPMC_CS0_BASE + nandcs * GPMC_CS_SIZE);
		blueshark_nand_data.gpmc_baseaddr = (void *) (gpmc_base_add);

		printk(KERN_INFO "Registering NAND on CS%d\n", nandcs);
		if (platform_device_register(&blueshark_nand_device) < 0)
			printk(KERN_ERR "Unable to register NAND device\n");
	}
}

static struct ehci_hcd_omap_platform_data ehci_pdata __initdata = {

	.port_mode[0] = EHCI_HCD_OMAP_MODE_PHY,
	.port_mode[1] = EHCI_HCD_OMAP_MODE_PHY,
	.port_mode[2] = EHCI_HCD_OMAP_MODE_UNKNOWN,

	.phy_reset  = true,
	.reset_gpio_port[0]  = -EINVAL,
	.reset_gpio_port[1]  = 20,
	.reset_gpio_port[2]  = -EINVAL
};

#ifdef CONFIG_OMAP_MUX
static struct omap_board_mux board_mux[] __initdata = {
	{ .reg_offset = OMAP_MUX_TERMINATOR },
};
#else
#define board_mux	NULL
#endif

static void __init blueshark_init(void)
{
	omap3_mux_init(board_mux, OMAP_PACKAGE_CUS);
	blueshark_i2c_init();
	platform_add_devices(blueshark_devices,
			ARRAY_SIZE(blueshark_devices));
	omap_serial_init();

	omap_mux_init_gpio(BLUESHARK_GPIO_DVI_EN, OMAP_PIN_INPUT);
	gpio_request(BLUESHARK_GPIO_DVI_EN, "DVI_nPD");
	/* REVISIT leave DVI powered down until it's needed ... */
	gpio_direction_output(BLUESHARK_GPIO_DVI_EN, 0);
	/* LCD backlight turn off */
	omap_mux_init_gpio(BLUESHARK_GPIO_LCD_BL, OMAP_PIN_INPUT);
	gpio_request(BLUESHARK_GPIO_LCD_BL, "LCD_BL");
	gpio_direction_output(BLUESHARK_GPIO_LCD_BL, 0);

	/* Assigning CD and WP pins for MMC2 */
	mmc[1].gpio_wp = 141;
	mmc[1].gpio_cd = 162;

	usb_musb_init();
	usb_ehci_init(&ehci_pdata);
	blueshark_flash_init();
	blueshark_spi_init();

	/* Ensure SDRC pins are mux'd for self-refresh */
	omap_mux_init_signal("sdrc_cke0", OMAP_PIN_OUTPUT);
	omap_mux_init_signal("sdrc_cke1", OMAP_PIN_OUTPUT);

	blueshark_display_init();
}
static void __init blueshark_map_io(void)
{
	omap2_set_globals_343x();
	omap2_map_common_io();
}

MACHINE_START(BLUESHARK, "OMAP3 BlueShark")
	/* Maintainer: Maxim Podbereznyy - http://mentorel.com */
	.phys_io	= 0x48000000,
	.io_pg_offst	= ((0xfa000000) >> 18) & 0xfffc,
	.boot_params	= 0x80000100,
	.map_io		= blueshark_map_io,
	.init_irq	= blueshark_init_irq,
	.init_machine	= blueshark_init,
	.timer		= &omap_timer,
MACHINE_END
