/*
 * LCD panel driver for Sharp LS037V7DW01
 *
 * Copyright (C) 2008 Nokia Corporation
 * Author: Tomi Valkeinen <tomi.valkeinen@nokia.com>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <linux/module.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/regulator/consumer.h>
#include <linux/err.h>

#include <plat/display.h>

static struct omap_video_timings osd102_timings = {
	.x_res = 1024,
	.y_res = 600,

	.pixel_clock	= 64000,

	.hsw		= 1,
	.hfp		= 160,
	.hbp		= 160,

	.vsw		= 1,
	.vfp		= 12,
	.vbp		= 23,
};

static int osd102_panel_probe(struct omap_dss_device *dssdev)
{
	dssdev->panel.config = OMAP_DSS_LCD_TFT;
	dssdev->panel.acb = 0;
	dssdev->panel.timings = osd102_timings;

	return 0;
}

static void osd102_panel_remove(struct omap_dss_device *dssdev)
{
}

static int osd102_panel_enable(struct omap_dss_device *dssdev)
{
	int r = 0;

	/* wait couple of vsyncs until enabling the LCD */
	msleep(50);

	if (dssdev->platform_enable)
		r = dssdev->platform_enable(dssdev);

	return r;
}

static void osd102_panel_disable(struct omap_dss_device *dssdev)
{
	if (dssdev->platform_disable)
		dssdev->platform_disable(dssdev);

	/* wait at least 5 vsyncs after disabling the LCD */

	msleep(100);
}

static int osd102_panel_suspend(struct omap_dss_device *dssdev)
{
	osd102_panel_disable(dssdev);
	return 0;
}

static int osd102_panel_resume(struct omap_dss_device *dssdev)
{
	return osd102_panel_enable(dssdev);
}

static struct omap_dss_driver osd102_driver = {
	.probe		= osd102_panel_probe,
	.remove		= osd102_panel_remove,

	.enable		= osd102_panel_enable,
	.disable	= osd102_panel_disable,
	.suspend	= osd102_panel_suspend,
	.resume		= osd102_panel_resume,

	.driver         = {
		.name   = "osd_osd102_panel",
		.owner  = THIS_MODULE,
	},
};

static int __init osd102_panel_drv_init(void)
{
	return omap_dss_register_driver(&osd102_driver);
}

static void __exit osd102_panel_drv_exit(void)
{
	omap_dss_unregister_driver(&osd102_driver);
}

module_init(osd102_panel_drv_init);
module_exit(osd102_panel_drv_exit);
MODULE_LICENSE("GPL");
